"use strict";

// istanbul ignore next

var _interopRequire = function (obj) { return obj && obj.__esModule ? obj["default"] : obj; };

// istanbul ignore next

var _slicedToArray = function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { var _arr = []; for (var _iterator = arr[Symbol.iterator](), _step; !(_step = _iterator.next()).done;) { _arr.push(_step.value); if (i && _arr.length === i) break; } return _arr; } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } };

// istanbul ignore next

var _toConsumableArray = function (arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } };

module.exports = quantityQueries;

var postcss = _interopRequire(require("postcss"));

var list = _interopRequire(require("postcss/lib/list"));

var balanced = _interopRequire(require("balanced-match"));

function quantityQueries() {
  return function (css) {
    css.eachRule(processRule);
    css.eachAtRule(processAtRule);
  };
}

var rePseudo = /(.*)(?::{1,2})(at-(?:least|most)|between|exactly)/;
var reAtRule = /(at-(?:least|most)|between|exactly)/;

function processRule(rule) {
  if (!rePseudo.test(rule.selector)) {
    return;
  }rule.selectors = rule.selectors.map(function (s) {
    var _balanced = balanced("(", ")", s);

    var pre = _balanced.pre;
    var body = _balanced.body;

    var args = list.comma(body);

    var _pre$split = pre.split(/:{1,2}/);

    var _pre$split2 = _slicedToArray(_pre$split, 2);

    var selector = _pre$split2[0];
    var quantifier = _pre$split2[1];

    return quantifiers[quantifier].apply(quantifiers, _toConsumableArray(args))([selector]);
  });
}

function processAtRule(atRule) {
  if (!reAtRule.test(atRule.name)) {
    return;
  }var args = list.space(atRule.params);
  var parent = atRule.parent;
  var root = parent.root();
  var selectors = quantifiers[atRule.name].apply(quantifiers, _toConsumableArray(args))(parent.selectors);

  var newRule = postcss.rule({
    selectors: selectors,
    nodes: atRule.nodes,
    source: atRule.source,
    semicolon: true
  });

  cleanIndent(newRule);

  root.insertAfter(parent, newRule);
  atRule.removeSelf();

  if (!parent.nodes.length) parent.removeSelf();
}

var cleanIndent = function (rule) {
  return rule.eachDecl(function (decl) {
    decl.before = decl.before.replace(/[^\S\x0a\x0d]{2,}/, "  ");
  });
};

var quantitySelectors = function (quantifier, last) {
  return function (selectors) {
    return selectors.map(function (s) {
      return "" + s + "" + quantifier + ", " + s + "" + quantifier + " ~ " + (last || list.space(s).pop());
    });
  };
};

var quantifiers = {

  "at-least": function (count, last) {
    return quantitySelectors(":nth-last-child(n+" + count + ")", last);
  },

  "at-most": function (count, last) {
    return quantitySelectors(":nth-last-child(-n+" + count + "):first-child", last);
  },

  between: function (start, end, last) {
    return quantitySelectors(":nth-last-child(n+" + start + "):nth-last-child(-n+" + end + "):first-child", last);
  },

  exactly: function (count, last) {
    return quantitySelectors(":nth-last-child(" + count + "):first-child", last);
  }

};